using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using Eve_Class_Library;
using EVE_Map_Importer.DBML;
using libEveMapNG.Data.Static;
using libEveMapNG.Types;
using libEveMapNG.Utility;
using Region = libEveMapNG.Data.Static.Region;
using RelativePosition = Eve_Class_Library.RelativePosition;

namespace EVE_Map_Importer
{
    internal class Program
    {
        private static readonly EVE_Data_DumpDataContext myContext = new EVE_Data_DumpDataContext();

        private static readonly int[] SkipRegions = new[]
                                                        {
                                                            10000004, 10000017, 10000019,
                                                            11000001, 11000002, 11000003, 11000004,
                                                            11000005, 11000006, 11000007, 11000008,
                                                            11000009, 11000010, 11000011, 11000012,
                                                            11000013, 11000014, 11000015, 11000016,
                                                            11000017, 11000018, 11000019, 11000020,
                                                            11000021, 11000022, 11000023, 11000024,
                                                            11000025, 11000026, 11000027, 11000028,
                                                            11000029, 11000030
                                                        };

        private static void Main()
        {
            //Trash the old data file before we start the import
            File.Delete("data.store");
            Console.WriteLine("-- Starting Import --");
            BuildEveUniverse();
            StaticStore.WriteToFile();
            Console.WriteLine("-- Data Store Written --");
            Console.ReadKey();
        }

        private static void BuildEveUniverse()
        {
            ImportStarbaseData();
            //ORDER IS IMPORTANT!
            BuildRegions();
            BuildConstellations();
            BuildSolarSystems();
            BuildPerSystemCelestialObjects();
            BuildStations();
            BuildAgents();
            //BuildFactions();
            //BuildNPCCorporations();
            BuildSolarSystemJumps();
            BuildStationTypeIDs();
            PreComputeConstellationJumpDriveAdjacency();
            //Build2DSolarSystemPoints();
            MergeOldEveMap2DCoordinates();
            SortPlanets();
        }

        private static void SortPlanets()
        {
            foreach (Region region in StaticStore.Instance.Regions)
            {
                foreach (Constellation constellation in region.Constellations)
                {
                    foreach (SolarSystem solarSystem in constellation.Systems)
                    {
                        solarSystem.Planets.OrderBy(p => p.Number);
                    }
                }
            }
        }

        private static void ImportStarbaseData()
        {
            var StarbaseTypes = from starbase in myContext.invTypes
                                join resourceConsumption in myContext.invControlTowerResources
                                    on starbase.typeID equals resourceConsumption.controlTowerTypeID
                                join resourceName in myContext.invTypes
                                    on resourceConsumption.resourceTypeID equals resourceName.typeID
                                where starbase.published == true
                                select
                                    new
                                        {
                                            TypeID = starbase.typeID,
                                            Name = starbase.typeName,
                                            FuelName = resourceName.typeName,
                                            FuelConsumption = resourceConsumption.quantity,
                                            Volume = resourceName.volume,
                                            FuelType = resourceName.typeID
                                        };

            Dictionary<int, Starbase> Types = new Dictionary<int, Starbase>();
            foreach (var type in StarbaseTypes)
            {
                if (!Types.ContainsKey(type.TypeID))
                {
                    Types.Add(type.TypeID, new Starbase());
                    Types[type.TypeID].Name = type.Name;
                    Types[type.TypeID].TypeID = type.TypeID;
                    Types[type.TypeID].Fuel = new Dictionary<string, StarbaseFuel>();
                }
                Starbase CurrentPosType = Types[type.TypeID];

                if (type.FuelConsumption == 0)
                    continue;

                StarbaseFuel CurrentFuel = new StarbaseFuel
                                               {
                                                   Name = type.FuelName,
                                                   QuantityUsedPerHour = ((int) type.FuelConsumption),
                                                   TypeId = type.FuelType,
                                                   Volume = ((float) type.Volume)
                                               };

                CurrentPosType.Fuel.Add(CurrentFuel.Name, CurrentFuel);
            }
            StaticStore.Instance.StarbaseTypes = Types;
        }

        private static void MergeOldEveMap2DCoordinates()
        {
            foreach (EVERegion region in EVEData.Instance.RegionList)
            {
                foreach (Region NGRegion in StaticStore.Instance.Regions)
                {
                    if (region.Name == NGRegion.Name)
                        NGRegion.Color = region.Color;
                }
                foreach (EVEConstellation constellation in region)
                {
                    foreach (Region NGRegion in StaticStore.Instance.Regions)
                    {
                        foreach (Constellation NGConstellation in NGRegion.Constellations)
                        {
                            if (constellation.Name == NGConstellation.Name)
                                NGConstellation.Color = constellation.Color;
                        }
                    }
                    foreach (EVESystem system in constellation)
                    {
                        StaticStore.Systems[system.ID].OrganicCoordinates = system.FlatCoords;

                        switch (system.FlatLabel)
                        {
                            case RelativePosition.Top:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.Top;
                                break;
                            case RelativePosition.TopLeft:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.TopLeft;
                                break;
                            case RelativePosition.TopRight:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.TopRight;
                                break;
                            case RelativePosition.CenterLeft:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.CenterLeft;
                                break;
                            case RelativePosition.CenterRight:
                                StaticStore.Systems[system.ID].FlatLabel =
                                    libEveMapNG.Types.RelativePosition.CenterRight;
                                break;
                            case RelativePosition.Bottom:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.Bottom;
                                break;
                            case RelativePosition.BottomLeft:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.BottomLeft;
                                break;
                            case RelativePosition.BottomRight:
                                StaticStore.Systems[system.ID].FlatLabel =
                                    libEveMapNG.Types.RelativePosition.BottomRight;
                                break;
                            case RelativePosition.Center:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.Center;
                                break;
                            case RelativePosition.None:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.None;
                                break;
                            default:
                                StaticStore.Systems[system.ID].FlatLabel = libEveMapNG.Types.RelativePosition.None;
                                break;
                        }
                    }
                }
            }
        }

        private static void BuildStationTypeIDs()
        {
            Console.Title = " -- Adding station type identifiers -- ";
            StaticStore.Instance.StationTypes = new Dictionary<int, StationType>();
            IQueryable<staStationType> TypeId = from type
                                                    in myContext.staStationTypes
                                                select type;
            StationType myType;
            foreach (staStationType type in TypeId)
            {
                myType = new StationType
                             {
                                 ID = type.stationTypeID ?? 0,
                                 OfficeSlots = (type.officeSlots ?? 0),
                                 ReprocessingEffeciency = (type.reprocessingEfficiency ?? 0)
                             };
                StaticStore.Instance.StationTypes.Add(myType.ID, myType);
                Console.WriteLine("Adding Type: {0}", myType.ID);
            }
        }

        private static void Build2DSolarSystemPoints()
        {
            foreach (KeyValuePair<int, SolarSystem> pair in StaticStore.Systems)
                pair.Value.OrganicCoordinates = new Point(Convert.ToInt32(Math.Round(pair.Value.Coordinates.X)),
                                                          Convert.ToInt32(Math.Round(pair.Value.Coordinates.Y)));
        }

        private static void PreComputeConstellationJumpDriveAdjacency()
        {
            foreach (Region region in StaticStore.Instance.Regions)
            {
                foreach (Constellation constellation in region.Constellations)
                {
                    if (constellation.JumpAdjacentConstellations == null)
                        constellation.JumpAdjacentConstellations = new List<int>();
                    foreach (SolarSystem system in constellation.Systems)
                    {
                        foreach (KeyValuePair<int, SolarSystem> pair in StaticStore.Systems)
                        {
                            if (pair.Value == system)
                                continue;
                            if (pair.Value.Security >= 0.45f)
                                continue;
                            if (JumpDrive.GetDistance(system, pair.Value) >= 14.625f)
                                continue;
                            if (!constellation.JumpAdjacentConstellations.Contains(pair.Value.ConstellationID))
                                constellation.JumpAdjacentConstellations.Add(pair.Value.ConstellationID);
                        }
                    }
                }
            }
        }

        private static void BuildPerSystemCelestialObjects()
        {
            Console.Title = " -- Importing Planets, Moons, Ice Fields, and Asteroid Belts --";
            foreach (KeyValuePair<int, SolarSystem> system in StaticStore.Systems)
            {
                var SystemObjects = from sysObject in myContext.mapDenormalizes
                                    join types in myContext.invTypes
                                        on sysObject.typeID equals types.typeID
                                    where sysObject.solarSystemID == (system.Value.ID)
                                    orderby sysObject.itemName
                                    select new {SystemObject = sysObject, Type = types};
                Planet CurrentPlanet = null;
                foreach (var sObject in SystemObjects)
                {
                    switch (sObject.Type.groupID)
                    {
                        case 6:
                            //star
                            //stars are always in the middle of the solar system
                            //dont bother importing
                            break;
                        case 7:
                            //planet
                            Planet newPlanet = new Planet
                                                   {
                                                       Number =
                                                           ushort.Parse(GetPlanetNumber(
                                                                            sObject.SystemObject.itemName.Replace(
                                                                                system.Value.Name, "")).ToString())
                                                   };
                            if (system.Value.Planets == null)
                                system.Value.Planets = new List<Planet>();
                            system.Value.Planets.Add(newPlanet);
                            CurrentPlanet = newPlanet;
                            break;
                        case 8:
                            //moon
                            if (CurrentPlanet == null)
                                break;
                            //Moon newMoon = new Moon
                            //                   {
                            //                       Number = MoonCounter
                            //                   };
                            //CurrentPlanet.Moons.Add(newMoon);
                            //MoonCounter++;
                            CurrentPlanet.Moons++;
                            break;
                        case 9:
                            //belts
                            if (CurrentPlanet == null)
                                break;
                            //Belt newBelt = new Belt
                            //                   {
                            //                       Number = BeltCounter
                            //                   };
                            switch (sObject.Type.typeID)
                            {
                                case 15:
                                    //newBelt.Type = BeltType.Asteroid;
                                    CurrentPlanet.Belts++;
                                    break;
                                case 17774:
                                    //newBelt.Type = BeltType.Ice;
                                    CurrentPlanet.IceFields++;
                                    break;
                                default:
                                    throw new Exception("Unknown Belt Type");
                            }
                            //CurrentPlanet.Belts.Add(newBelt);
                            //BeltCounter++;
                            break;
                        case 10:
                            //stargate
                            break;
                        case 15:
                            //station
                            break;
                        default:
                            throw new Exception("Unknown Object");
                    }
                }
            }
        }

        private static void BuildSolarSystemJumps()
        {
            Console.Title = " -- Importing Solar System MapJumps --";
            IQueryable<mapSolarSystemJump> myGates = from stargates in myContext.mapSolarSystemJumps
                                                     where
                                                         //Remove Jove Regions
                                                         !SkipRegions.Contains((int) stargates.fromRegionID)
                                                         &&
                                                         !SkipRegions.Contains((int) stargates.toRegionID)
                                                     select stargates;
            foreach (mapSolarSystemJump gate in myGates)
            {
                StaticStore.Instance.StarGates = new List<StarGate>();
                StarGate newGate = new StarGate
                                       {
                                           From = SolarSystem.GetByID(gate.fromSolarSystemID),
                                           To = SolarSystem.GetByID(gate.toSolarSystemID)
                                       };
                if (gate.fromRegionID != gate.toRegionID)
                    newGate.Type = GateType.InterRegion;
                else if (gate.fromConstellationID != gate.toConstellationID)
                    newGate.Type = GateType.InterConstellation;
                else
                    newGate.Type = GateType.Normal;
                SolarSystem fromSolarSystem =
                    SolarSystem.GetByID(gate.fromSolarSystemID);
                if (fromSolarSystem.StarGates == null)
                    fromSolarSystem.StarGates = new List<StarGate>();
                fromSolarSystem.StarGates.Add(newGate);
                SolarSystem toSolarSystem = SolarSystem.GetByID(gate.toSolarSystemID);
                if (toSolarSystem.StarGates == null)
                    toSolarSystem.StarGates = new List<StarGate>();
                toSolarSystem.StarGates.Add(newGate);
                Console.WriteLine("Gate - {0} <-> {1}", gate.fromSolarSystemID, gate.toSolarSystemID);
            }
            CleanupStargates();
        }

        private static void CleanupStargates()
        {
            //Builds a list of stargates
            foreach (Region region2 in StaticStore.Instance.Regions)
            {
                if (region2.Constellations == null)
                    continue;
                foreach (Constellation constellation2 in region2.Constellations)
                {
                    foreach (SolarSystem system2 in constellation2.Systems)
                    {
                        if (system2.StarGates != null)
                        {
                            foreach (StarGate stargate in system2.StarGates)
                                StaticStore.Instance.StarGates.Add(stargate);
                        }
                    }
                }
            }

            //Removes duplicate gates from the store.
            for (int i = StaticStore.Instance.StarGates.Count - 1; i >= 0; i--)
            {
                StarGate curGate = StaticStore.Instance.StarGates[i];
                for (int j = StaticStore.Instance.StarGates.Count - 1; j >= 0; j--)
                {
                    StarGate destGate = StaticStore.Instance.StarGates[j];
                    if (curGate.From == destGate.To &&
                        curGate.To == destGate.From)
                    {
                        StaticStore.Instance.StarGates.RemoveAt(i);

                        break;
                    }
                }
            }
        }

        private static void BuildSolarSystems()
        {
            Console.Title = "-- Importing Solar Systems --";
            IQueryable<mapSolarSystem> systems = from p in myContext.mapSolarSystems
                                                 where
                                                     !SkipRegions.Contains((int) p.regionID)
                                                 select p;
            foreach (mapSolarSystem system in systems)
            {
                Console.WriteLine("- {0}", system.solarSystemName);
                SolarSystem newSystem = new SolarSystem
                                            {
                                                Name = system.solarSystemName,
                                                ID = system.solarSystemID,
                                                ConstellationID = (int) system.constellationID,
                                                RegionID = (int) system.regionID,
                                                Security = ((double) system.security),
                                                SecurityClass = system.securityClass,
                                                Coordinates =
                                                    new PointF3D((float) system.x, (float) system.z,
                                                                 (float) system.y)
                                            };
                if (Constellation.GetByID(newSystem.ConstellationID).Systems == null)
                    Constellation.GetByID(newSystem.ConstellationID).Systems = new List<SolarSystem>();
                Constellation.GetByID(newSystem.ConstellationID).Systems.Add(newSystem);
            }
            StaticStore.RebuildHashTables();
        }

        private static void BuildConstellations()
        {
            Console.Title = "-- Importing Constellations --";
            IQueryable<mapConstellation> constellations = from c in myContext.mapConstellations
                                                          where !SkipRegions.Contains((int) c.regionID)
                                                          select c;
            foreach (mapConstellation constellation in constellations)
            {
                Console.WriteLine("> {0}", constellation.constellationName);
                Constellation newConstellation = new Constellation
                                                     {
                                                         ID = constellation.constellationID,
                                                         Name = constellation.constellationName,
                                                         RegionID = (int) constellation.regionID
                                                     };
                if (Region.GetByID(newConstellation.RegionID).Constellations == null)
                    Region.GetByID(newConstellation.RegionID).Constellations = new List<Constellation>();
                Region.GetByID(newConstellation.RegionID).Constellations.Add(newConstellation);
            }
            StaticStore.RebuildHashTables();
        }

        private static void BuildRegions()
        {
            StaticStore.Instance.Regions = new List<Region>();
            IQueryable<mapRegion> regions = from r in myContext.mapRegions
                                            where !SkipRegions.Contains(r.regionID)
                                            select r;
            Console.Title = "--  Importing Regions --";
            foreach (mapRegion region in regions)
            {
                Console.WriteLine("# {0}", region.regionName);
                Region newRegion = new Region
                                       {
                                           ID = region.regionID,
                                           Name = region.regionName
                                       };
                StaticStore.Instance.Regions.Add(newRegion);
            }
            StaticStore.RebuildHashTables();
        }

        private static void BuildStations()
        {
            Console.Title = "-- Importing Stations --";
            IQueryable<staStation> myStations = from stations in myContext.staStations
                                                where
                                                    //avoid conquerables
                                                    !(stations.stationTypeID == 21646 || stations.stationTypeID == 21645 ||
                                                      stations.stationTypeID == 21644 || stations.stationTypeID == 21642 ||
                                                      stations.stationTypeID == 12295 || stations.stationTypeID == 12294 ||
                                                      stations.stationTypeID == 12242)
                                                select stations;
            myStations = from s in myStations
                         where
                             !SkipRegions.Contains((int) s.regionID)
                         select s;
            //Preload the station services to speed up importing stations
            var myServices = new Dictionary<int, List<string>>();
            var OperationServices = from opSvc in myContext.staOperationServices
                                    join Svcs in myContext.staServices
                                        on opSvc.serviceID equals Svcs.serviceID
                                    //Throw out various station services we don't care about
                                    where !(Svcs.serviceID == 524288 || Svcs.serviceID == 4 ||
                                            Svcs.serviceID == 262144 || Svcs.serviceID == 524288 ||
                                            Svcs.serviceID == 524288 || Svcs.serviceID == 33554432 ||
                                            Svcs.serviceID == 64 || Svcs.serviceID == 2097152 ||
                                            Svcs.serviceID == 67108864)
                                    select new {OperationID = opSvc.operationID, Service = Svcs.serviceName};
            foreach (var service in OperationServices)
            {
                if (!myServices.Keys.Contains(service.OperationID))
                    myServices.Add(service.OperationID, new List<string> {service.Service});
                else
                    myServices[service.OperationID].Add(service.Service);
            }
            foreach (staStation station in myStations)
            {
                Station newStation = new Station
                                         {
                                             ID = station.stationID,
                                             SolarSystemID = ((int) station.solarSystemID),
                                             Name = station.stationName,
                                             Refine = ((double) station.reprocessingEfficiency),
                                             Services = myServices[station.operationID],
                                             Faction = ((int) station.corporationID)
                                         };
                Console.WriteLine(station.stationName);
                SolarSystem solarSystem =
                    SolarSystem.GetByID((int) station.solarSystemID);
                if (solarSystem.Stations == null)
                    solarSystem.Stations = new List<Station>();
                solarSystem.Stations.Add(newStation);
            }
            StaticStore.RebuildHashTables();
            Console.WriteLine(StaticStore.Stations.Count());
        }

        private static void BuildAgents()
        {
            Console.Title = " -- Importing Agents --";
            var Agents = from agents in myContext.agtAgents
                         join agenttype in myContext.agtAgentTypes
                             on agents.agentTypeID equals agenttype.agentTypeID
                         join division in myContext.crpNPCDivisions
                             on agents.divisionID equals division.divisionID
                         join agentname in myContext.eveNames
                             on agents.agentID equals agentname.itemID
                         join s in myContext.staStations
                             on agents.stationID equals s.stationID
                         //Agents in Jove Space.  We really should give them the finger
                         where !SkipRegions.Contains((int) s.regionID)
                         select new
                                    {
                                        Agent = agents,
                                        AgentType = agenttype,
                                        Division = division,
                                        Name = agentname
                                    };
            //Preload locator agents
            List<int> locatorAgents = (from config in myContext.agtConfigs
                                       where
                                           config.k.StartsWith("agent.Locate")
                                       select config.agentID).ToList();
            foreach (var agent in Agents)
            {
                if (agent.Agent.stationID == null)
                {
                    /* If the agent doesn't live in a station
                     * we have no way to figure out where the agent is...
                     * so we give him/her the finger 
                     * 
                     * ......................//) 
                     * ....................,/../ 
                     * .................../..../ 
                     * .............//'...'/` 
                     * ........../'/.../..../......./\ 
                     * ........('(.......... ~/'...') 
                     * .........\.................'...../ 
                     * ..........''...\.......... _. 
                     * ............\..............( 
                     * ..............\.............\... 
                     * 
                     * and don't import him
                     */
                    continue;
                }
                Console.WriteLine("Agent: {0}", agent.Name.itemName);
                Agent newAgent = new Agent
                                     {
                                         Type = agent.AgentType.agentType,
                                         ID = agent.Agent.agentID,
                                         Level = Int16.Parse(agent.Agent.level.ToString()),
                                         Quality = agent.Agent.quality ?? 0,
                                         Division = agent.Division.divisionName,
                                         Name = agent.Name.itemName,
                                         StationID = (int) agent.Agent.stationID
                                     };

                if (locatorAgents.Contains(agent.Agent.agentID))
                    newAgent.Locator = true;
                int station = (int) agent.Agent.stationID;
                if (Station.GetByID(station).Agents == null)
                    Station.GetByID(station).Agents = new List<Agent>();
                Station.GetByID(station).Agents.Add(newAgent);
            }
        }

        /*static void BuildFactions()
        {
            Console.Title = " -- Importing Factions --";
            foreach (var faction in myContext.chrFactions)
            {
                Console.WriteLine(faction.factionName);
                Faction newFaction = new Faction
                                         {
                                             Name = faction.factionName,
                                             ID = faction.factionID
                                         };
                myStore.Factions.Add(newFaction);
            }
        }

        static void BuildNPCCorporations()
        {
            Console.Title = " -- Import NPC Corporations --";
            foreach (var corporation in myContext.crpNPCCorporations)
            {
                NPCCorporation newCorp = new NPCCorporation
                                             {
                                                 ID = corporation.corporationID,
                                                 FactionID = ((int)corporation.factionID)
                                             };
                myStore.NPCCorporations.Add(newCorp);
            }
        }*/

        private static int GetPlanetNumber(string s)
        {
            int lastSpace;
            if (s.Contains("("))
            {
                string cutPlanetName = s.Substring(0, s.IndexOf(" ("));
                lastSpace = GetLastSpacePosition(cutPlanetName);
                return RomanToNumber(cutPlanetName.Substring(lastSpace + 1,
                                                             cutPlanetName.Length - lastSpace - 1));
            }
            lastSpace = GetLastSpacePosition(s);
            return RomanToNumber(s.Substring(lastSpace + 1, s.Length - lastSpace - 1));
        }

        // Converts a Roman numerals value into an integer
        public static int RomanToNumber(string roman)
        {
            // Rule 7
            roman = roman.ToUpper().Trim();
            if (roman == "N")
                return 0;
            // Rule 4
            if (roman.Split('V').Length > 2 ||
                roman.Split('L').Length > 2 ||
                roman.Split('D').Length > 2)
                throw new ArgumentException("Rule 4");
            // Rule 1
            int count = 1;
            char last = 'Z';
            foreach (char numeral in roman)
            {
                // Valid character?
                // CCP probably using something dumb
                if ("IVXLCDM".IndexOf(numeral) == -1)
                    return 0;
                // Duplicate?
                if (numeral == last)
                {
                    count++;
                    if (count == 4)
                        throw new ArgumentException("Rule 1");
                }
                else
                {
                    count = 1;
                    last = numeral;
                }
            }
            // Create an ArrayList containing the values
            int ptr = 0;
            var values = new List<int>();
            int maxDigit = 1000;
            while (ptr < roman.Length)
            {
                // Base value of digit
                char numeral = roman[ptr];
                int digit = (int) Enum.Parse(typeof (RomanDigit), numeral.ToString());
                // Rule 3
                if (digit > maxDigit)
                    throw new ArgumentException("Rule 3");
                // Next digit
                int nextDigit;
                if (ptr < roman.Length - 1)
                {
                    char nextNumeral = roman[ptr + 1];
                    nextDigit = (int) Enum.Parse(typeof (RomanDigit), nextNumeral.ToString());
                    if (nextDigit > digit)
                    {
                        if ("IXC".IndexOf(numeral) == -1 ||
                            nextDigit > (digit*10) ||
                            roman.Split(numeral).Length > 3)
                            throw new ArgumentException("Rule 3");
                        maxDigit = digit - 1;
                        digit = nextDigit - digit;
                        ptr++;
                    }
                }
                values.Add(digit);
                // Next digit
                ptr++;
            }
            // Rule 5
            for (int i = 0; i < values.Count - 1; i++)
            {
                if (values[i] < values[i + 1])
                    throw new ArgumentException("Rule 5");
            }
            // Rule 2
            int total = 0;
            foreach (int digit in values)
                total += digit;
            return total;
        }

        private static int GetLastSpacePosition(string s)
        {
            for (int i = s.Length - 1; i >= 0; i--)
            {
                if (s[i] == ' ')
                    return i;
            }
            return -1;
        }

        #region Nested type: RomanDigit

        private enum RomanDigit
        {
            I = 1,
            V = 5,
            X = 10,
            L = 50,
            C = 100,
            D = 500,
            M = 1000
        }

        #endregion
    }
}