﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using EVEMapNG.SubForms;
using EveMapSettings;
using libEveMapNG.Data;
using libEveMapNG.Data.Static;

namespace EVEMapNG
{
    /// <summary>Inits EVE-Map</summary>
    internal static class Program
    {
        /// <summary>
        ///   The main entry point for the application.
        /// </summary>
        [STAThread]
        private static void Main()
        {
            //Global Unhandled Exception... handler setup
            AppDomain.CurrentDomain.UnhandledException += CurrentDomainUnhandledException;
            Application.SetUnhandledExceptionMode(UnhandledExceptionMode.ThrowException);

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            SplashScreen splashScreen = new SplashScreen();
            splashScreen.Show();

            LoadUserSettings();

            Loader.LoadAllData();
            Application.OpenForms["SplashScreen"].Close();

            Application.Run(new MainWindow());
        }

        private static void LoadUserSettings()
        {
            try
            {
                UserSettings.Read();
            }
            catch
            {
                UserSettings.Instance = new UserSettings();
            }

            if (UserSettings.Instance.FirstRun || UserSettings.Instance.AppState == null)
            {
                SetupDefaults();

                UserSettings.Instance.FirstRun = false;
            }
        }

        private static void SetupDefaults()
        {
            UserSettings.Instance.AppState = new ApplicationState();
            UserSettings.Instance.AppState.LoadDefaultSettings();

            //Set goonfleet default API locations
            UserSettings.Instance.RestJumpbridgeUrl = "http://apps.goonfleet.com/bridgeplanner/api/1/bridge-list/";
            UserSettings.Instance.RestCynoBeaconUrl = "http://apps.goonfleet.com/bridgeplanner/api/1/cyno-list/";

            //Setup default color schemes
            MapColorScheme myColorScheme = new MapColorScheme
                                               {
                                                   Name = "Light",
                                                   BackgroundColor = Color.White,
                                                   SystemTextColor = Color.Black,
                                                   CynoBeaconColor = Color.FromArgb(55, 55, 85),
                                                   GateGenericColor = Color.FromArgb(43, 71, 196),
                                                   GateConstellationColor = Color.FromArgb(114, 63, 117),
                                                   GateRegionColor = Color.FromArgb(196, 43, 71),
                                                   GateRouteColor = Color.FromArgb(43, 196, 71),
                                                   JumpBridgeRouteColor = Color.FromArgb(163, 98, 40),
                                                   JumpRouteColor = Color.FromArgb(0, 128, 255),
                                                   MinimapBackgroundColor = Color.GhostWhite,
                                                   MinimapBorderColor = Color.Black,
                                                   MinimapFocusBackgroundColor = Color.FromArgb(0x30, Color.DarkRed),
                                                   MinimapFocusOutlineColor = Color.Red,
                                                   HighSecDetailColor = Color.Blue,
                                                   LowSecDetailColor = Color.Yellow,
                                                   NullSecDetailColor = Color.DarkRed
                                               };
            UserSettings.Instance.AddMapColorScheme(myColorScheme);


            myColorScheme = new MapColorScheme
                                {
                                    Name = "Dark",
                                    BackgroundColor = Color.Black,
                                    SystemTextColor = Color.White,
                                    CynoBeaconColor = Color.FromArgb(190, 190, 220),
                                    GateGenericColor = Color.FromArgb(43, 71, 196),
                                    GateConstellationColor = Color.FromArgb(114, 63, 117),
                                    GateRegionColor = Color.FromArgb(196, 43, 71),
                                    GateRouteColor = Color.FromArgb(43, 196, 71),
                                    JumpBridgeRouteColor = Color.FromArgb(163, 98, 40),
                                    JumpRouteColor = Color.FromArgb(163, 98, 40),
                                    MinimapBackgroundColor = Color.FromArgb(30, 30, 30),
                                    MinimapBorderColor = Color.FromArgb(50, 50, 50),
                                    MinimapFocusBackgroundColor = Color.FromArgb(0x80, Color.FromArgb(60, 30, 30)),
                                    MinimapFocusOutlineColor = Color.Red,
                                    HighSecDetailColor = Color.FromArgb(118, 161, 71),
                                    LowSecDetailColor = Color.FromArgb(196, 135, 71),
                                    NullSecDetailColor = Color.FromArgb(196, 43, 71)
                                };

            UserSettings.Instance.AddMapColorScheme(myColorScheme);

            //Setup default viewport settings
            ViewportSettings mySettings = new ViewportSettings
                                              {
                                                  Name = "Default",
                                                  DetailTextFontFamily = "Tahoma",
                                                  MapLabelFontFamily = "Verdana",
                                                  SystemLabelFontFamily = "Tahoma",
                                                  Margin = new Size(400, 400),
                                                  LineThickness = 2,
                                                  SystemMarkerMargin = 2,
                                                  SystemMarkerDiameter = 4,
                                                  ShowRegionNames = true,
                                                  ShowConstNames = false,
                                                  ShowSystemNames = true,
                                                  ShowSystemDetails = true,
                                                  ShowJumpbridges = true,
                                                  ShowCynoGenerators = true,
                                                  ShowSovCircles = false,
                                                  ShowMiniMap = true,
                                                  DetailText = EVEMapDetailText.None
                                              };
            UserSettings.Instance.AddUserViewportStyle(mySettings);
            UserSettings.Instance.SetActiveColorScheme("Light");
        }

        private static void CurrentDomainUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            if (!Debugger.IsAttached)
            {
                ShowError(e.ExceptionObject as Exception);
                Environment.Exit(1);
            }
        }

        private static void ShowError(Exception e)
        {
            using (UnhandledExceptionWindow f = new UnhandledExceptionWindow(e))
                f.ShowDialog();
        }
    }
}