using System.Drawing;
using System.Windows.Forms;
using EveLib.Types;

namespace EveLib.Renderer
{
    public abstract class BaseLayer
    {
        protected static RectangleF ubounds;
        protected Point margin = Point.Empty;
        protected float scale = 1.0f;
        protected PointF scrollOffset = PointF.Empty;

        protected BaseLayer()
        {
            Disabled = false;
            if (ubounds.Size == SizeF.Empty)
                ubounds = Universe.Bounds;
        }

        public Control Parent { get; set; }
        public bool Disabled { get; set; }

        /// <summary>
        /// Scale of the Viewport
        /// </summary>
        public float Scale
        {
            get { return scale; }
            set { scale = value; }
        }

        /// <summary>
        /// Offset from the scrollbars
        /// </summary>
        public PointF ScrollOffset
        {
            get { return scrollOffset; }
            set { scrollOffset = value; }
        }

        /// <summary>
        /// Margin that offsets all points
        /// </summary>
        public Point Margin
        {
            get { return margin; }
            set { margin = value; }
        }

        /// <summary>
        /// Effective size of the viewport
        /// </summary>
        public Size EffectiveSize { get; set; }

        /// <summary>
        /// Get the scaled bounds of the whole universe
        /// </summary>
        /// <returns></returns>
        public RectangleF Bounds
        {
            get { return ScaleRectangle(ubounds); }
        }

        public float LineWidth
        {
            get { return 2.5f*((Scale < .4f) ? .4f : Scale); }
        }

        #region Scaling

        /// <summary>
        /// Lowest point in the scaling process.
        /// </summary>
        /// <param name="f"></param>
        /// <returns></returns>
        public virtual float ScaleFloat(float f)
        {
            return f*scale;
        }

        /// <summary>
        /// Scales a float in the opposite direction (large zoom means smaller return)
        /// </summary>
        /// <param name="f"></param>
        /// <returns></returns>
        public virtual float ScaleFloatInverse(float f)
        {
            return f*(1f - scale);
        }

        /// <summary>
        /// For scaling X-axis numbers (adds margins and scroll offset)
        /// </summary>
        /// <param name="x"></param>
        /// <returns></returns>
        public virtual float ScaleX(float x)
        {
            return ScaleFloat(x - ubounds.Left) + Margin.X - scrollOffset.X;
        }

        /// <summary>
        /// For scaling X-axis numbers (adds margins and scroll offset)
        /// </summary>
        /// <param name="y"></param>
        /// <returns></returns>
        public virtual float ScaleY(float y)
        {
            return ScaleFloat(y - ubounds.Top) + Margin.Y - scrollOffset.Y;
        }

        /// <summary>
        /// For scaling widths (adds the margin)
        /// </summary>
        /// <param name="width"></param>
        /// <returns></returns>
        public virtual float ScaleWidth(float width)
        {
            return ScaleFloat(width) + Margin.X;
        }

        /// <summary>
        /// For scaling heights (adds the margin)
        /// </summary>
        /// <param name="height"></param>
        /// <returns></returns>
        public virtual float ScaleHeight(float height)
        {
            return ScaleFloat(height) + Margin.Y;
        }

        /// <summary>
        /// Scale a 2D point in space
        /// </summary>
        /// <param name="p">A 2D point in space</param>
        /// <returns>A scaled 2D point in space</returns>
        public PointF ScalePoint(PointF p)
        {
            p.X = ScaleX(p.X);
            p.Y = ScaleY(p.Y);
            return p;
        }

        /// <summary>
        /// Scale a 2D point in space and add the universe offset
        /// </summary>
        /// <param name="p">A 2D point in space</param>
        /// <returns>A scaled 2D point in space</returns>
        public PointF ScalePointOffset(PointF p)
        {
            p.X = ScaleFloat(p.X - ubounds.Left);
            p.Y = ScaleFloat(p.Y - ubounds.Top);
            return p;
        }

        /// <summary>
        /// Converts a 3D point in space to 2D and scales it
        /// </summary>
        /// <param name="p">A 3D point in space</param>
        /// <returns>A scaled 2D point in space</returns>
        public PointF ScalePoint(PointF3D p)
        {
            return new PointF(ScaleX(p.X), ScaleY(p.Y));
        }

        /// <summary>
        /// Scale a Size
        /// </summary>
        /// <param name="p"></param>
        /// <returns></returns>
        public SizeF ScaleSize(SizeF p)
        {
            p.Width = ScaleWidth(p.Width);
            p.Height = ScaleHeight(p.Height);
            return p;
        }

        /// <summary>
        /// Scale a Rectangle
        /// </summary>
        /// <param name="r"></param>
        /// <returns></returns>
        public RectangleF ScaleRectangle(RectangleF r)
        {
            PointF coords = ScalePoint(new PointF(r.Left, r.Top));
            SizeF dimensions = ScaleSize(new SizeF(r.Width, r.Height));
            return new RectangleF(coords, dimensions);
        }

        #endregion

        public abstract void Draw(Graphics g, RectangleF c);
    }
}