using System;
using System.Drawing;
using libEveMapNG.Data.Static;

namespace libEveMapNG.Renderer.Text
{
    public interface ISolarSystemColorScheme
    {
        Color GetSystemColor(SolarSystem system);
        Brush GetSystemBrush(SolarSystem system);
        Pen GetSystemPen(SolarSystem system);
        Pen GetSystemPen(SolarSystem system, float width);
    }

    public class SolarSystemColorSchemes
    {
        private static readonly SecurityColorScheme securityScheme = new SecurityColorScheme();

        public static SecurityColorScheme SecurityScheme
        {
            get { return securityScheme; }
        }
    }

    public abstract class AbstractSolarSystemColorScheme : ISolarSystemColorScheme
    {
        // IMPLEMENT THIS IN DERIVED CLASSES

        #region ISolarSystemColorScheme Members

        public abstract Color GetSystemColor(SolarSystem system);

        public virtual Brush GetSystemBrush(SolarSystem system)
        {
            return new SolidBrush(GetSystemColor(system));
        }

        public virtual Pen GetSystemPen(SolarSystem system)
        {
            return new Pen(GetSystemColor(system));
        }

        public virtual Pen GetSystemPen(SolarSystem system, float width)
        {
            return new Pen(GetSystemColor(system), width);
        }

        #endregion
    }

    public abstract class PartitionedColorScheme : AbstractSolarSystemColorScheme
    {
        protected SolidBrush[] brushes;
        protected Color[] colors;
        protected Pen[] floatPens;
        protected Pen[] pens;

        private float width = -1;

        protected PartitionedColorScheme() : this(1, 1)
        {
        }

        protected PartitionedColorScheme(float defaultWidth) : this(defaultWidth, 1)
        {
        }

        protected PartitionedColorScheme(float defaultWidth, float initialVariableWidth)
        {
            LoadColors();
            Width = width;
            brushes = new SolidBrush[colors.Length];
            pens = new Pen[colors.Length];
            for (int i = 0; i < colors.Length; ++i)
            {
                brushes[i] = new SolidBrush(colors[i]);
                pens[i] = new Pen(colors[i], defaultWidth);
            }
        }

        /// <summary>
        ///   Sets the floating pen to this value
        /// </summary>
        /// <remarks>
        ///   This will not affect default drawing operations with pens, only the GetFloatingSystemPen() call & cache
        /// </remarks>
        public float Width
        {
            get { return width; }
            set
            {
                if (value == width) return;
                if (floatPens != null)
                {
                    foreach (Pen pen in floatPens)
                        if (pen != null)
                            pen.Dispose();
                }
                floatPens = new Pen[colors.Length];
                width = value;
            }
        }

        protected abstract void LoadColors();

        protected abstract Object GetSystemIndex(SolarSystem system);

        public override Brush GetSystemBrush(SolarSystem system)
        {
            return (Brush) GetSystemObject(system, brushes);
        }

        public override Pen GetSystemPen(SolarSystem system)
        {
            return (Pen) GetSystemObject(system, pens);
        }

        public override Color GetSystemColor(SolarSystem system)
        {
            Object index = GetSystemIndex(system);
            if (index != null && index is int)
                return colors[(int) index];
            throw new ArgumentException("System has no value");
        }

        public override Pen GetSystemPen(SolarSystem system, float width)
        {
            Width = width;
            int index = (int) GetSystemIndex(system);
            if (floatPens[index] == null)
                floatPens[index] = new Pen(colors[index], width);
            return floatPens[index];
        }

        public Pen GetFloatingSystemPen(SolarSystem system)
        {
            return GetSystemPen(system, width);
        }

        protected Object GetSystemObject(SolarSystem system, Object list)
        {
            Object index = GetSystemIndex(system);
            if (index != null && index is int)
                return ((object[]) list)[(int) index];
            throw new ArgumentException("System has no value");
        }

        protected Color GetSystemColor(float ratio)
        {
            ratio *= colors.Length;
            return colors[Convert.ToInt32(Math.Round(ratio, 3))];
        }
    }
}